@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/python2
 
 import math
 import os
@@ -25,7 +25,7 @@ class EditableEntry (Gtk.Stack):
 
     __gsignals__ = {
         'changed': (GObject.SignalFlags.RUN_FIRST, None,
-                      (str,))
+                    (str,))
     }
 
     def __init__ (self):
@@ -92,7 +92,7 @@ class EditableEntry (Gtk.Stack):
     def get_text(self):
         return self.entry.get_text()
 
-class SidePage:
+class SidePage(object):
     def __init__(self, name, icon, keywords, content_box = None, size = None, is_c_mod = False, is_standalone = False, exec_name = None, module=None):
         self.name = name
         self.icon = icon
@@ -300,9 +300,11 @@ class SettingsStack(Gtk.Stack):
         self.expand = True
 
 class SettingsRevealer(Gtk.Revealer):
-    def __init__(self, schema=None, key=None, values=None):
+    def __init__(self, schema=None, key=None, values=None, check_func=None):
         Gtk.Revealer.__init__(self)
 
+        self.check_func = check_func
+
         self.box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=15)
         Gtk.Revealer.add(self, self.box)
 
@@ -311,10 +313,9 @@ class SettingsRevealer(Gtk.Revealer):
 
         if schema:
             self.settings = Gio.Settings.new(schema)
-            #the value of this key is the information whether to show or to hide the revealer
-            if values is None:
+            # if there aren't values or a function provided to determine visibility we can do a simple bind
+            if values is None and check_func is None:
                 self.settings.bind(key, self, "reveal-child", Gio.SettingsBindFlags.GET)
-            #only at some values of this key the reveaer must be shown
             else:
                 self.values = values
                 self.settings.connect("changed::" + key, self.on_settings_changed)
@@ -325,7 +326,11 @@ class SettingsRevealer(Gtk.Revealer):
 
     #only used when checking values
     def on_settings_changed(self, settings, key):
-        self.set_reveal_child(settings.get_value(key).unpack() in self.values)
+        value = settings.get_value(key).unpack()
+        if self.check_func is None:
+            self.set_reveal_child(value in self.values)
+        else:
+            self.set_reveal_child(self.check_func(value, self.values))
 
 class SettingsPage(Gtk.Box):
     def __init__(self):
@@ -408,7 +413,7 @@ class SettingsBox(Gtk.Frame):
 
         self.need_separator = True
 
-    def add_reveal_row(self, widget, schema=None, key=None, values=None):
+    def add_reveal_row(self, widget, schema=None, key=None, values=None, check_func=None):
         vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         if self.need_separator:
             vbox.add(Gtk.Separator(orientation=Gtk.Orientation.HORIZONTAL))
@@ -420,7 +425,7 @@ class SettingsBox(Gtk.Frame):
             list_box.connect("row-activated", widget.clicked)
         list_box.add(row)
         vbox.add(list_box)
-        revealer = SettingsRevealer(schema, key, values)
+        revealer = SettingsRevealer(schema, key, values, check_func)
         widget.revealer = revealer
         revealer.add(vbox)
         self.box.add(revealer)
@@ -467,6 +472,18 @@ class SettingsWidget(Gtk.Box):
             settings_objects[schema] = Gio.Settings.new(schema)
             return settings_objects[schema]
 
+class SettingsLabel(Gtk.Label):
+    def __init__(self, text=None):
+        Gtk.Label.__init__(self)
+        if text:
+            self.set_label(text)
+
+        self.set_alignment(0.0, 0.5)
+        self.set_line_wrap(True)
+
+    def set_label_text(self, text):
+        self.set_label(text)
+
 class IndentedHBox(Gtk.HBox):
     def __init__(self):
         super(IndentedHBox, self).__init__()
@@ -486,8 +503,8 @@ class Switch(SettingsWidget):
     def __init__(self, label, dep_key=None, tooltip=""):
         super(Switch, self).__init__(dep_key=dep_key)
 
-        self.content_widget = Gtk.Switch()
-        self.label = Gtk.Label(label)
+        self.content_widget = Gtk.Switch(valign=Gtk.Align.CENTER)
+        self.label = SettingsLabel(label)
         self.pack_start(self.label, False, False, 0)
         self.pack_end(self.content_widget, False, False, 0)
 
@@ -507,7 +524,7 @@ class SpinButton(SettingsWidget):
 
         if units:
             label += " (%s)" % units
-        self.label = Gtk.Label.new(label)
+        self.label = SettingsLabel(label)
         self.content_widget = Gtk.SpinButton()
 
         self.pack_start(self.label, False, False, 0)
@@ -555,8 +572,9 @@ class Entry(SettingsWidget):
     def __init__(self, label, expand_width=False, size_group=None, dep_key=None, tooltip=""):
         super(Entry, self).__init__(dep_key=dep_key)
 
-        self.label = Gtk.Label.new(label)
+        self.label = SettingsLabel(label)
         self.content_widget = Gtk.Entry()
+        self.content_widget.set_valign(Gtk.Align.CENTER)
 
         self.pack_start(self.label, False, False, 0)
         self.pack_end(self.content_widget, expand_width, expand_width, 0)
@@ -601,9 +619,10 @@ class FontButton(SettingsWidget):
     def __init__(self, label, size_group=None, dep_key=None, tooltip=""):
         super(FontButton, self).__init__(dep_key=dep_key)
 
-        self.label = Gtk.Label.new(label)
+        self.label = SettingsLabel(label)
 
         self.content_widget = Gtk.FontButton()
+        self.content_widget.set_valign(Gtk.Align.CENTER)
 
         self.pack_start(self.label, False, False, 0)
         self.pack_end(self.content_widget, False, False, 0)
@@ -617,7 +636,7 @@ class Range(SettingsWidget):
     bind_prop = "value"
     bind_dir = Gio.SettingsBindFlags.GET | Gio.SettingsBindFlags.NO_SENSITIVITY
 
-    def __init__(self, label, min_label="", max_label="", mini=None, maxi=None, step=None, invert=False, log=False, dep_key=None, tooltip=""):
+    def __init__(self, label, min_label="", max_label="", mini=None, maxi=None, step=None, invert=False, log=False, show_value=True, dep_key=None, tooltip="", flipped=False):
         super(Range, self).__init__(dep_key=dep_key)
 
         self.set_orientation(Gtk.Orientation.VERTICAL)
@@ -625,6 +644,7 @@ class Range(SettingsWidget):
 
         self.log = log
         self.invert = invert
+        self.flipped = flipped
         self.timer = None
         self.value = 0
 
@@ -653,8 +673,20 @@ class Range(SettingsWidget):
         if log:
             mini = math.log(mini)
             maxi = math.log(maxi)
-            self.map_get = lambda x: math.log(x)
-            self.map_set = lambda x: math.exp(x)
+            if self.flipped:
+                self.map_get = lambda x: -1 * (math.log(x))
+                self.map_set = lambda x: math.exp(x)
+            else:
+                self.map_get = lambda x: math.log(x)
+                self.map_set = lambda x: math.exp(x)
+        elif self.flipped:
+            self.map_get = lambda x: x * -1
+            self.map_set = lambda x: x * -1
+
+        if self.flipped:
+            tmp_mini = mini
+            mini = maxi * -1
+            maxi = tmp_mini * -1
 
         if step is None:
             self.step = (maxi - mini) * 0.02
@@ -663,7 +695,7 @@ class Range(SettingsWidget):
 
         self.content_widget = Gtk.Scale.new_with_range(Gtk.Orientation.HORIZONTAL, mini, maxi, self.step)
         self.content_widget.set_inverted(invert)
-        self.content_widget.set_draw_value(False)
+        self.content_widget.set_draw_value(show_value and not self.flipped)
         self.bind_object = self.content_widget.get_adjustment()
 
         if invert:
@@ -684,9 +716,12 @@ class Range(SettingsWidget):
     def apply_later(self, *args):
         def apply(self):
             if self.log:
-                self.set_value(math.exp(self.content_widget.get_value()))
+                self.set_value(math.exp(abs(self.content_widget.get_value())))
             else:
-                self.set_value(self.content_widget.get_value())
+                if self.flipped:
+                    self.set_value(self.content_widget.get_value() * -1)
+                else:
+                    self.set_value(self.content_widget.get_value())
             self.timer = None
 
         if self.timer:
@@ -720,7 +755,7 @@ class ComboBox(SettingsWidget):
         self.valtype = valtype
         self.option_map = {}
 
-        self.label = Gtk.Label.new(label)
+        self.label = SettingsLabel(label)
 
         selected = None
 
@@ -731,6 +766,7 @@ class ComboBox(SettingsWidget):
 
         self.pack_start(self.label, False, False, 0)
         self.pack_end(self.content_widget, False, False, 0)
+        self.content_widget.set_valign(Gtk.Align.CENTER)
 
         self.set_options(options)
 
@@ -776,7 +812,7 @@ class ColorChooser(SettingsWidget):
         # still support it for now by adding the legacy_string argument
         self.legacy_string = legacy_string
 
-        self.label = Gtk.Label(label)
+        self.label = SettingsLabel(label)
         self.content_widget = Gtk.ColorButton()
         self.content_widget.set_use_alpha(True)
         self.pack_start(self.label, False, False, 0)
@@ -813,7 +849,7 @@ class FileChooser(SettingsWidget):
         else:
             action = Gtk.FileChooserAction.OPEN
 
-        self.label = Gtk.Label(label)
+        self.label = SettingsLabel(label)
         self.content_widget = Gtk.FileChooserButton(action=action)
         self.pack_start(self.label, False, False, 0)
         self.pack_end(self.content_widget, False, False, 0)
@@ -838,7 +874,7 @@ class SoundFileChooser(SettingsWidget):
     def __init__(self, label, size_group=None, dep_key=None, tooltip=""):
         super(SoundFileChooser, self).__init__(dep_key=dep_key)
 
-        self.label = Gtk.Label(label)
+        self.label = SettingsLabel(label)
         self.content_widget = Gtk.Box()
 
         c = self.content_widget.get_style_context()
@@ -860,7 +896,7 @@ class SoundFileChooser(SettingsWidget):
         self.pack_end(self.content_widget, False, False, 0)
 
         self.play_button = Gtk.Button()
-        self.play_button.set_image(Gtk.Image.new_from_stock("gtk-media-play", Gtk.IconSize.BUTTON))
+        self.play_button.set_image(Gtk.Image.new_from_icon_name("media-playback-start-symbolic", Gtk.IconSize.BUTTON))
         self.play_button.connect("clicked", self.on_play_clicked)
         self.content_widget.pack_start(self.play_button, False, False, 0)
 
@@ -868,7 +904,7 @@ class SoundFileChooser(SettingsWidget):
 
         try:
             Gio.DBusProxy.new_for_bus(Gio.BusType.SESSION, Gio.DBusProxyFlags.NONE, None,
-                                      'org.cinnamon.SettingsDaemon',
+                                      'org.cinnamon.SettingsDaemon.Sound',
                                       '/org/cinnamon/SettingsDaemon/Sound',
                                       'org.cinnamon.SettingsDaemon.Sound',
                                       None, self._on_proxy_ready, None)
@@ -911,9 +947,9 @@ class SoundFileChooser(SettingsWidget):
         dialog.destroy()
 
     def update_button_label(self, absolute_path):
-        f = Gio.File.new_for_path(absolute_path)
-
-        self.button_label.set_label(f.get_basename())
+        if absolute_path != "":
+            f = Gio.File.new_for_path(absolute_path)
+            self.button_label.set_label(f.get_basename())
 
     def on_setting_changed(self, *args):
         self.update_button_label(self.get_value())
@@ -930,7 +966,7 @@ class IconChooser(SettingsWidget):
 
         valid, self.width, self.height = Gtk.icon_size_lookup(Gtk.IconSize.BUTTON)
 
-        self.label = Gtk.Label.new(label)
+        self.label = SettingsLabel(label)
 
         self.content_widget = Gtk.Box()
         self.bind_object = Gtk.Entry()
@@ -962,11 +998,11 @@ class IconChooser(SettingsWidget):
             self.preview.set_from_icon_name(val, Gtk.IconSize.BUTTON)
 
     def on_button_pressed(self, widget):
-        dialog = Gtk.FileChooserDialog(_("Choose an Icon"),
-                                           None,
-                                           Gtk.FileChooserAction.OPEN,
-                                           (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                                            Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
+        dialog = Gtk.FileChooserDialog(title=_("Choose an Icon"),
+                                       action=Gtk.FileChooserAction.OPEN,
+                                       transient_for=self.get_toplevel(),
+                                       buttons=(_("_Cancel"), Gtk.ResponseType.CANCEL,
+                                                _("_Open"), Gtk.ResponseType.OK))
 
         filter_text = Gtk.FileFilter()
         filter_text.set_name(_("Image files"))
@@ -982,22 +1018,23 @@ class IconChooser(SettingsWidget):
         if response == Gtk.ResponseType.OK:
             filename = dialog.get_filename()
             self.bind_object.set_text(filename)
-            self.set_val(filename)
+            self.set_value(filename)
 
         dialog.destroy()
 
     def update_icon_preview_cb(self, dialog, preview):
         filename = dialog.get_preview_filename()
         dialog.set_preview_widget_active(False)
-        if os.path.isfile(filename):
-            pixbuf = GdkPixbuf.Pixbuf.new_from_file(filename)
-            if pixbuf is not None:
-                if pixbuf.get_width() > 128:
-                    pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(filename, 128, -1)
-                elif pixbuf.get_height() > 128:
-                    pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(filename, -1, 128)
-                preview.set_from_pixbuf(pixbuf)
-                dialog.set_preview_widget_active(True)
+        if filename is not None:
+            if os.path.isfile(filename):
+                pixbuf = GdkPixbuf.Pixbuf.new_from_file(filename)
+                if pixbuf is not None:
+                    if pixbuf.get_width() > 128:
+                        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(filename, 128, -1)
+                    elif pixbuf.get_height() > 128:
+                        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(filename, -1, 128)
+                    preview.set_from_pixbuf(pixbuf)
+                    dialog.set_preview_widget_active(True)
 
 class TweenChooser(SettingsWidget):
     bind_prop = "tween"
@@ -1006,7 +1043,7 @@ class TweenChooser(SettingsWidget):
     def __init__(self, label, size_group=None, dep_key=None, tooltip=""):
         super(TweenChooser, self).__init__(dep_key=dep_key)
 
-        self.label = Gtk.Label.new(label)
+        self.label = SettingsLabel(label)
 
         self.content_widget = TweenChooserButton()
 
@@ -1025,7 +1062,7 @@ class EffectChooser(SettingsWidget):
     def __init__(self, label, possible=None, size_group=None, dep_key=None, tooltip=""):
         super(EffectChooser, self).__init__(dep_key=dep_key)
 
-        self.label = Gtk.Label.new(label)
+        self.label = SettingsLabel(label)
 
         self.content_widget = EffectChooserButton(possible)
 
@@ -1043,7 +1080,7 @@ class DateChooser(SettingsWidget):
     def __init__(self, label, size_group=None, dep_key=None, tooltip=""):
         super(DateChooser, self).__init__(dep_key=dep_key)
 
-        self.label = Gtk.Label.new(label)
+        self.label = SettingsLabel(label)
 
         self.content_widget = DateChooserButton()
 
@@ -1074,12 +1111,13 @@ class Keybinding(SettingsWidget):
 
         self.num_bind = num_bind
 
-        self.label = Gtk.Label(label)
+        self.label = SettingsLabel(label)
 
         self.buttons = []
         self.teach_button = None
 
         self.content_widget = Gtk.Frame(shadow_type=Gtk.ShadowType.IN)
+        self.content_widget.set_valign(Gtk.Align.CENTER)
         box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
         self.content_widget.add(box)
 
